#!/bin/bash

set -e

SERIES=utopic
CLICK_CHROOT_NAME=click
CLICK_SDK_ARCH=${CLICK_SDK_ARCH:=armhf}
CLICK_SDK_FRAMEWORK=${CLICK_SDK_FRAMEWORK:=ubuntu-sdk-14.10}
TG_PLUGIN=${TG_PLUGIN:=lp:libqtelegram}

echo -e "\033[1;32m [ ${CLICK_SDK_ARCH} ] Bootstrapping development environment for Telegram\033[0m"

get_plugin() {
    echo -e "           \033[1;32mBranching\033[1;33m ${TG_PLUGIN}\033[1;32m and updating library files in the source tree\033[0m"

    rm -rf library
    bzr branch $TG_PLUGIN library
}

update_chroot() {
    echo -e "\033[0;31m [ ${CLICK_SDK_ARCH} ] Updating click chroot\033[0m"

    click chroot -a ${CLICK_SDK_ARCH} -f ${CLICK_SDK_FRAMEWORK} -n ${CLICK_CHROOT_NAME} upgrade
    click chroot -a ${CLICK_SDK_ARCH} -f ${CLICK_SDK_FRAMEWORK} -n ${CLICK_CHROOT_NAME} maint apt-get install -y \
    libssl-dev:${CLICK_SDK_ARCH} \
    libsqlite3-dev:${CLICK_SDK_ARCH} \
    libmediainfo-dev:${CLICK_SDK_ARCH} \
    libthumbnailer-dev:${CLICK_SDK_ARCH} \
    pkg-config-arm-linux-gnueabihf \
    qtdeclarative5-ubuntu-telephony-phonenumber0.1 \
    intltool \
    bzr \
    --no-install-recommends
}

create_chroot() {
    echo -e "\033[1;31m [ ${CLICK_SDK_ARCH} ] Creating click chroot " \
        "${CLICK_SDK_FRAMEWORK}-${CLICK_SDK_ARCH} and installing dependencies\033[0m"
    sudo /usr/share/qtcreator/ubuntu/scripts/click_create_target \
            ${CLICK_SDK_ARCH} ${CLICK_SDK_FRAMEWORK} $SERIES $CLICK_CHROOT_NAME
    update

}

get_plugin

if ! click chroot -a ${CLICK_SDK_ARCH} -f ${CLICK_SDK_FRAMEWORK} -n ${CLICK_CHROOT_NAME} exists; then
    create_chroot
fi

if [ "$1" == "update" ]; then
    update_chroot
fi

echo -e "\033[1;32m [ ${CLICK_SDK_ARCH} ] Bootstrapping complete.\033[0m"
