import QtQuick 2.3
import QtQuick.Layouts 1.1
import Ubuntu.Components 1.1

Item {
    id: audioBox

    width: units.gu(28)
    height: units.gu(8)

    property bool isPlaying: audioPlayer.playing && audioPlayer.filePath == audio

    signal clicked(var mouse)

    RowLayout {
        anchors {
            top: parent.top
            left: parent.left
            right: parent.right
            bottom: timeLabel.top
            rightMargin: units.gu(1)
            leftMargin: units.gu(1)
        }
        spacing: units.gu(1)

        Item {
            anchors {
                top: parent.top
                bottom: parent.bottom
                margins: units.gu(1)
            }
            width: height

            Icon {
                id: playIcon
                anchors.fill: parent

                visible: !needsDownload
                color: textColor
                name: isPlaying ? "media-playback-pause" : "media-playback-start"
            }

            Image {
                anchors {
                    fill: parent
                }
                visible: needsDownload
                source: {
                    if (needsDownload) {
                        if (isDownloading) {
                            return Qt.resolvedUrl("../images/photocancel.png");
                        } else {
                            return Qt.resolvedUrl("../images/photoload.png");
                        }
                    }
                    return ""
                }
            }

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    if (isPlaying) {
                        audioPlayer.pause();
                    } else {
                        audioPlayer.stop();
                        audioPlayer.filePath = audio;
                        audioPlayer.play();
                    }
                    audioBox.clicked(mouse);
                }
            }
        }

        Slider {
            id: slider
            Layout.fillWidth: true
            anchors {
                verticalCenter: parent.verticalCenter
                margins: units.gu(1)
            }
            value: 0
            minimumValue: 0
            maximumValue: 100

            Connections {
                // we need to manually set the binding between value and progress,
                // otherwise Slider won't work
                target: audioPlayer
                onPositionChanged: {
                    if (isPlaying && audioPlayer.duration > 0) {
                        slider.value = 100 * audioPlayer.position / audioPlayer.duration;
                    } else {
                        slider.value = 0;
                    }
                }
            }
        }
    }

    Label {
        id: timeLabel
        anchors {
            bottom: parent.bottom
            left: parent.left
            leftMargin: units.gu(1)
            bottomMargin: units.dp(4)
        }
        text: isPlaying ?
            delegateUtils.formatTime(audioPlayer.position, true) + " / " + delegateUtils.formatTime(audioDuration)
            : delegateUtils.formatTime(audioDuration)
        color: textColor
        fontSize: userSettings.contents.fontSize
    }
}
