import QtQuick 2.0
import Telegram 0.1

Item {
    function getMediaTypeString(mediaType) {
        switch (mediaType) {
        case TLMessageMedia.TypeMessageMediaEmpty:
            return "Text";
        case TLMessageMedia.TypeMessageMediaPhoto:
            return "Photo";
        case TLMessageMedia.TypeMessageMediaVideo:
            return "Video";
        case TLMessageMedia.TypeMessageMediaGeo:
            return "Location";
        case TLMessageMedia.TypeMessageMediaContact:
            return "Contact";
        case TLMessageMedia.TypeMessageMediaAudio:
            return "Audio";
        case TLMessageMedia.TypeMessageMediaDocument:
            return "Document";
        case TLMessageMedia.TypeMessageMediaUnsupported:
            return "Unsupported";
        default:
            return "Text";
        }
    }

    function getActionMessageText(outgoing, senderName, actionType,
                                  actionUser, actionTitle, actionThumbnail) {
        actionUser = actionUser.trim();
        var displayName = senderName;
        if (outgoing) displayName = i18n.tr("You");

        if (actionUser == "") {
            // TODO karni: We should be still able to display deleted user's name.
            // Why is a DELETED user's actionUser == " "?
            actionUser = i18n.tr("DELETED");
        }

        switch (true) {
        case actionType == TLMessageAction.TypeMessageActionEmpty:
            return "";
        case actionType == TLMessageAction.TypeMessageActionChatCreate: {
            return i18n.tr("%1 created %2").arg(displayName).arg(actionTitle);
        }
        case actionType == TLMessageAction.TypeMessageActionChatAddUser: {
            if (senderName == actionUser) {
                return i18n.tr("%1 rejoined the group").arg(displayName);
            } else {
                return i18n.tr("%1 added %2").arg(displayName).arg(actionUser);
            }
        }
        case actionType == TLMessageAction.TypeMessageActionChatDeleteUser: {
            if (senderName == actionUser) {
                return i18n.tr("%1 left the group").arg(displayName);
            } else {
                return i18n.tr("%1 removed %2").arg(displayName).arg(actionUser);
            }
        }
        case actionType == TLMessageAction.TypeMessageActionChatEditTitle:
            return i18n.tr("%1 changed group name to %2").arg(displayName).arg(actionTitle);
        case actionType == TLMessageAction.TypeMessageActionChatEditPhoto:
            return i18n.tr("%1 changed the group photo").arg(displayName); // TODO actionThumbnail
        case actionType == TLMessageAction.TypeMessageActionChatDeletePhoto:
            return i18n.tr("%1 removed the group photo").arg(displayName);
        default:
            console.log("Unhandled action: " + actionType);
            return "";
        }
    }

    function getSecretChatState(state, isOutgoing) {
        switch (state) {
        case 0: return i18n.tr("Waiting for user to get online...");
        case 1: return i18n.tr("Secret chat requested");
        case 2: return i18n.tr("Secret chat created");
        case 3: return i18n.tr("Secret chat cancelled");
        default: return "";
        }
    }

    function getSecretChatSelfDestructTime(seconds) {
        if (seconds === 0) {
            return i18n.tr("Disabled");
        } else if (seconds > 0 && seconds <= 30) {
            return i18n.tr("%1 second", "%1 seconds", seconds).arg(seconds);
        } else if (seconds === 60) {
            return i18n.tr("1 minute");
        } else if (seconds === 3600) {
            return i18n.tr("1 hour");
        } else if (seconds === 86400) {
            return i18n.tr("1 day");
        } else if (seconds === 604800) {
            return i18n.tr("1 week");
        }
    }

    function humanFileSize(bytes, si) {
        var thresh = si ? 1000 : 1024;
        if (bytes < thresh) return bytes + ' B';
        var units = si ? ['kB','MB','GB','TB','PB','EB','ZB','YB'] : ['KiB','MiB','GiB','TiB','PiB','EiB','ZiB','YiB'];
        var u = -1;
        do {
            bytes /= thresh;
            ++u;
        } while(bytes >= thresh);
        return bytes.toFixed(1) + ' ' + units[u];
    }

    function formatTime(time, ms) {
        if (ms) {
            // convert ms to s
            time = Math.floor(time / 1000);
        }
        var mm = Math.floor(time / 60);
        var ss = time - (mm * 60);
        return mm.toString() + ":" + ("0" + ss).slice(-2);
    }
}
