#include <QDebug>
#include <QFile>
#include <QJsonDocument>
#include <QJsonArray>
#include <QStringList>
#include <QTextStream>

#include "pushhelper.h"

PushHelper::PushHelper(QString appId, QString infile, QString outfile,
                       QObject *parent) : QObject(parent) {
    connect(&mPushClient, SIGNAL(persistentCleared()),
                    this, SLOT(notificationDismissed()));

    mPushClient.setAppId(appId);
    mPushClient.registerApp(appId);
    mInfile = infile;
    mOutfile = outfile;
}

PushHelper::~PushHelper() {
}

void PushHelper::process() {
    QString tag = "";

    QJsonObject pushMessage = readPushMessage(mInfile);
    mPostalMessage = pushToPostalMessage(pushMessage, tag);
    if (!tag.isEmpty()) {
        dismissNotification(tag);
    }
}

void PushHelper::notificationDismissed() {
    writePostalMessage(mPostalMessage, mOutfile);
    Q_EMIT done();
}

QJsonObject PushHelper::readPushMessage(const QString &filename) {
    QFile file(filename);
    file.open(QIODevice::ReadOnly | QIODevice::Text);

    QString val = file.readAll();
    file.close();
    return QJsonDocument::fromJson(val.toUtf8()).object();
}

void PushHelper::writePostalMessage(const QJsonObject &postalMessage, const QString &filename) {
    QFile out;
    out.setFileName(filename);
    out.open(QIODevice::WriteOnly | QIODevice::Text | QIODevice::Truncate);

    QTextStream(&out) << QJsonDocument(postalMessage).toJson();
    out.close();
}

QJsonObject PushHelper::pushToPostalMessage(const QJsonObject &push, QString &tag) {
    QString summary = "";
    QString body = "";
    qint32 count = 0;

    QJsonObject message = push["message"].toObject();
    QJsonObject custom = message["custom"].toObject();

    QString key = "";
    if (message.keys().contains("loc_key")) {
        key = message["loc_key"].toString();
    }
    QJsonArray args;
    if (message.keys().contains("loc_args")) {
        args = message["loc_args"].toArray();
    }

    QString chatId = "0"; // More useful as string in this context.
    if (custom.keys().contains("from_id")) {
        chatId = custom["from_id"].toString();
    }
    if (custom.keys().contains("chat_id")) {
        chatId = custom["chat_id"].toString();
    }

    QString tg = QString("Telegram");
    summary = args[0].toString();

    if (key == "MESSAGE_TEXT") {

        body = args[1].toString();

    } else if (key == "MESSAGE_NOTEXT") {

        body = "sent you a message";

    } else if (key == "MESSAGE_PHOTO") {

        body = "sent you a photo";

    } else if (key == "MESSAGE_VIDEO") {

        body = "sent you a video";

    } else if (key == "MESSAGE_DOC") {

        body = "sent you a document";

    } else if (key == "MESSAGE_AUDIO") {

        body = "sent you a voice message";

    } else if (key == "MESSAGE_CONTACT") {

        body = "shared a contact with you";

    } else if (key == "MESSAGE_GEO") {

        body = "sent you a map";

    } else if (key == "CHAT_MESSAGE_TEXT") {

        summary = args[1].toString();
        body = args[0].toString() + ": " + args[2].toString();

    } else if (key == "CHAT_MESSAGE_NOTEXT") {

        summary = args[1].toString();
        body = args[0].toString() + " sent a message to the group";

    } else if (key == "CHAT_MESSAGE_PHOTO") {

        summary = args[1].toString();
        body = args[0].toString() + " sent a photo to the group";

    } else if (key == "CHAT_MESSAGE_VIDEO") {

        summary = args[1].toString();
        body = args[0].toString() + " sent a video to the group";

    } else if (key == "CHAT_MESSAGE_DOC") {

        summary = args[1].toString();
        body = args[0].toString() + " sent a document to the group";

    } else if (key == "CHAT_MESSAGE_AUDIO") {

        summary = args[1].toString();
        body = args[0].toString() + " sent a voice message to the group";

    } else if (key == "CHAT_MESSAGE_CONTACT") {

        summary = args[1].toString();
        body = args[0].toString() + " sent a contact to the group";

    } else if (key == "CHAT_MESSAGE_GEO") {

        summary = args[1].toString();
        body = args[0].toString() + " sent a map to the group";

    } else if (key == "CHAT_CREATED") {

        summary = args[1].toString();
        body = args[0].toString() + " invited you to the group";

    } else if (key == "CHAT_TITLE_EDITED") {

        summary = args[1].toString();
        body = args[0].toString() + " changed group name";

    } else if (key == "CHAT_PHOTO_EDITED") {

        summary = args[1].toString();
        body = args[0].toString() + " changed group photo";

    } else if (key == "CHAT_ADD_MEMBER") {

        summary = args[1].toString();
        body = args[0].toString() + " invited " + args[2].toString();

    } else if (key == "CHAT_ADD_YOU") {

        summary = args[1].toString();
        body = args[0].toString() + " invited you to the group";

    } else if (key == "CHAT_DELETE_MEMBER") {

        summary = args[1].toString();
        body = args[0].toString() + " kicked " + args[2].toString();

    } else if (key == "CHAT_DELETE_YOU") {

        summary = args[1].toString();
        body = args[0].toString() + " kicked you from the group";

    } else if (key == "CHAT_LEFT") {

        summary = args[1].toString();
        body = args[0].toString() + " has left the group";

    } else if (key == "CHAT_RETURNED") {

        summary = args[1].toString();
        body = args[0].toString() + " has returned to the group";

    } else if (key == "GEOCHAT_CHECKIN") {

        summary = "@ " + args[1].toString();
        body = args[0].toString() + " has checked-in";

    } else if (key == "CONTACT_JOINED") {

        summary = tg;
        body = args[0].toString() + " joined Telegram!";

    } else if (key == "AUTH_UNKNOWN") {

        summary = args[0].toString();
        body = "New login from unrecognized device";

    } else if (key == "AUTH_REGION") {

        summary = args[0].toString() + " @ " + args[1].toString();
        body = "New login from unrecognized device";

    } else if (key == "CONTACT_PHOTO") {

        body = "updated profile photo";

    } else if (key == "ENCRYPTION_REQUEST") {

        summary = tg;
        body = "You have a new message";

    } else if (key == "ENCRYPTION_ACCEPT") {

        summary = tg;
        body = "You have a new message";

    } else if (key == "ENCRYPTED_MESSAGE") {

        summary = tg;
        body = "You have a new message";

    } else {
        qDebug() << "Unhandled push type: " << key;
        return QJsonObject();
    }

    QJsonArray actions = QJsonArray();
    QString actionUri = QString("telegram://chat/%1").arg(chatId);
    actions.append(actionUri);

    if (message.keys().contains("badge")) {
        count = message["badge"].toInt();
    } else if (push.keys().contains("notification")) {
        // Legacy. Notification section is only used to retrieve the unread count.
        count = push["notification"]
                .toObject()["emblem-counter"]
                .toObject()["count"]
                .toInt();
    }

    tag = chatId;

    QJsonObject card;
    card["summary"] = summary;
    card["body"]    = body;
    card["actions"] = actions;
    card["popup"]   = true;             // TODO make setting
    card["persist"] = true;             // TODO make setting

    QJsonObject emblem;
    emblem["count"] = count;
    emblem["visible"] = count > 0;

    QJsonObject notification;
    notification["tag"] = tag;
    notification["card"] = card;
    notification["emblem-counter"] = emblem;
    notification["sound"] = true;       // TODO make setting
    notification["vibrate"] = true;     // TODO make setting

    QJsonObject postalMessage = QJsonObject();
    postalMessage["notification"] = notification;

    return postalMessage;
}

void PushHelper::dismissNotification(const QString &tag) {
    QStringList tags;
    tags << tag;
    mPushClient.clearPersistent(tags);
}
