/*
 * Copyright 2015 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import Telegram 0.1
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0 as ListItem
import Ubuntu.Components.Popups 0.1

import "../components"
import "../js/avatar.js" as Avatar
import "../js/time.js" as Time

TelegramPage {
    id: blockedUsersPage

    pageTitle: i18n.tr("Blocked Users")

    head.actions: [
        Action {
            iconName: "add"
            text: i18n.tr("Block contact")
            enabled: isConnected
            onTriggered: addBlockedContactPressed()
        }
    ]

    body: ListView {
        anchors.fill: parent

        highlightFollowsCurrentItem: false
        clip: true

        model: blockedUsersModel
        delegate: TelegramContactsListItem {
            id: delegate

            userId: model.id
            photo: model.photo
            title: model.name
            isOnline: model.online
            lastSeen: model.lastSeenOnline

            onItemClicked: {
                mouse.accepted = true;
                if (model.id !== telegramClient.ourId) {
                    openProfile(model.id);
                }
            }

            leftSideActions: [
                Action {
                    iconName: "remove"
                    text: i18n.tr("Unblock contact")
                    visible: isConnected
                    onTriggered: {
                        PopupUtils.open(Qt.resolvedUrl("dialogs/ConfirmationDialog.qml"),
                            delegate, {
                                text: i18n.tr("Unblock this user?"),
                                onAccept: function() {
                                    inputPeer.userId = model.id;
                                    telegramClient.unblockContact(inputPeer);
                                }
                            }
                        );
                    }
                }
            ]
        }
    }



    BlockedUsersModel {
        id: blockedUsersModel
    }

    TLInputPeer {
        id: inputPeer
    }

    Component.onCompleted: {
        blockedUsersModel.setup(telegramClient);
    }

    function addBlockedContactPressed() {
        var properties = { blockUserMode: true };
        pageStack.push(contactsPage, properties);
    }
}
