import QtQuick 2.0
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0
import Ubuntu.Components.Popups 1.0
import "../components"
import "../components/listitems"
import "../js/time.js" as Time
import Telegram 0.1

TelegramPage {
    id: dialogsPage

    property var messagesToForward: []

    head.actions: [
            Action {
                iconName: "search"
                text: i18n.tr("Search")
                visible: !isSearching
                onTriggered: searchPressed()
            },
            Action {
                iconName: "compose"
                text: i18n.tr("Compose")
                enabled: isConnected
                onTriggered: newChat()
            },
            Action {
                iconName: "contact-group"
                text: i18n.tr("New Group")
                enabled: isConnected
                onTriggered: newGroupChat()
            },
            Action {
                iconName: "contact"
                text: i18n.tr("Contacts")
                onTriggered: newChat()
            },
            Action {
                iconName: "settings"
                text: i18n.tr("Settings")
                onTriggered: openSettings()
            }
        ]

    head.backAction.visible: isSearching || messagesToForward.length > 0

    body: Item {
        anchors.fill: parent

        Label {
            id: listEmptyLabel
            anchors.centerIn: parent
            fontSize: "medium"
            visible: dialogsListView.count === 0
            text: !isOnline ? i18n.tr("Disconnected")
                            : isConnected ? i18n.tr("No chats") : i18n.tr("Waiting for connection...")
            z: 0
        }

        DialogsProxy {
            id: dialogsProxy
            sourceModel: dialogsModel
            sortRole: DialogsModel.DateRole
            dynamicSortFilter: true
            ascending: false
            searchTerm: dialogsPage.searchTerm
        }

        Rectangle {
            id: shareContentHintRect
            anchors {
                top: parent.top
                left: parent.left
                right: parent.right
            }
            height: units.gu(visible ? 4 : 0)
            visible: transferHelper.hasContent
            color: UbuntuColors.orange

            Icon {
                id: shareContentHintButton
                anchors {
                    top: parent.top
                    topMargin: units.dp(4)
                    right: parent.right
                    rightMargin: units.gu(2)
                    bottom: parent.bottom
                    bottomMargin: units.dp(4)
                }
                width: height
                color: "#F3BEAC"
                name: "close"

                MouseArea {
                    anchors.fill: parent
                    onClicked: transferHelper.cancel()
                }
            }

            Label {
                id: shareContentHintLabel
                anchors {
                    top: parent.top
                    left: parent.left
                    bottom: parent.bottom
                    right: shareContentHintButton.left
                }
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                color: "white"
                fontSize: "medium"
                text: i18n.tr("Select chat to share content")
            }
        }

        ListView {
            id: dialogsListView
            anchors {
                top: shareContentHintRect.bottom
                left: parent.left
                right: parent.right
                bottom: parent.bottom
            }
            clip: true
            z: 1

            cacheBuffer: units.gu(8)*20
            model: dialogsProxy
            delegate: TelegramDialogsListItem {
                id: dialogsListItem
                // this avoid to forward messages to secret chats, which don't support such
                // feature
                height: messagesToForward.length > 0 && isSecret ? 0 : units.gu(8)

                dialogId: model.id
                peerId: model.peerId
                peerType: model.peerType

                isSecret: model.isSecret
                isOutgoing: model.topMessageOut
                isSent: model.topMessageSent
                isRead: !model.topMessageUnread
                isAction: model.topMessageIsAction ||
                          (model.isSecret && model.topMessageId === 0)

                actionType: model.topMessageActionType
                actionTitle: model.topMessageActionTitle
                actionUser: model.topMessageActionUser
                state: model.state
                topMessageId: model.topMessageId

                isTyping: model.typing
                whoIsTyping: model.whoIsTyping
                message: model.topMessageText
                thumbnail: model.thumbnail
                mediaType: model.topMessageMediaType

                title: model.name
                senderName: model.topMessageFromFirstName
                unreadCount: model.unreadCount
                messageDate: model.date

                onItemClicked: {
                    mouse.accepted = true;
                    searchFinished();
                    var properties = {};
                    if (transferHelper.hasContent) {
                        PopupUtils.open(Qt.resolvedUrl("dialogs/ConfirmationDialog.qml"), 
                            dialogsListItem, {
                                text: i18n.tr("Share content with %1?".arg(title)),
                                onAccept: function() {
                                    properties['shareContent'] = true;
                                    openChatById(dialogId, properties);
                                },
                                altText: i18n.tr("Select different chat"),
                                onCancel: function() {
                                    transferHelper.cancel();
                                }
                            }
                        );
                    } else if (messagesToForward.length > 0) {
                        PopupUtils.open(Qt.resolvedUrl("dialogs/ConfirmationDialog.qml"), 
                            dialogsListItem, {
                                text: i18n.tr("Forward message to %1?".arg(title)),
                                onAccept: function() {
                                    properties['messagesToForward'] = messagesToForward;
                                    openChatById(dialogId, properties);
                                    messagesToForward = [];
                                }
                            }
                        );
                    } else {
                        openChatById(dialogId, properties);
                        messagesToForward = [];
                    }
                }

                leftSideActions: [
                    Action {
                        iconName: "system-log-out"
                        text: i18n.tr("Delete chat")
                        visible: isConnected
                        onTriggered: {
                            console.log("action: delete chat");
                            PopupUtils.open(Qt.resolvedUrl("dialogs/ConfirmationDialog.qml"), 
                                dialogsListItem, {
                                    text: i18n.tr("Are you sure you want to delete this chat?"),
                                    onAccept: function() {
                                        if (isSecret) {
                                            telegramClient.discardSecretChat(dialogId);
                                            telegramClient.deleteSecretChatHistory(dialogId, true);
                                        } else {
                                            inputPeer.setupPeer(peerId, peerType);
                                            telegramClient.deleteChatHistory(inputPeer, 0, true);
                                        }
                                    }
                                }
                            );
                        }
                    },
                    Action {
                        iconName: "edit-clear"
                        text: i18n.tr("Clear history")
                        visible: isConnected
                        onTriggered: {
                            console.log("action: clear chat");
                            PopupUtils.open(Qt.resolvedUrl("dialogs/ConfirmationDialog.qml"), 
                                dialogsListItem, {
                                    text: i18n.tr("Are you sure you want to clear history?"),
                                    onAccept: function() {
                                        inputPeer.setupPeer(peerId, peerType);
                                        if (isSecret) {
                                            telegramClient.deleteSecretChatHistory(dialogId, false);
                                        } else {
                                            telegramClient.deleteChatHistory(inputPeer, 0, false);
                                        }
                                    }
                                }
                            );
                        }
                    }
                ]

                rightSideActions: [
                    Action {
                        iconName: "info"
                        text: i18n.tr("Info")
                        onTriggered: {
                            if (TLInputPeer.TypeInputPeerChat === model.peerType) {
                                openGroupProfile(peerId);
                            } else {
                                openProfile(peerId);
                            }
                        }
                    }
                ]
            }

            Component.onCompleted: {
                // FIXME: workaround for qtubuntu not returning values depending on the grid unit definition
                // for Flickable.maximumFlickVelocity and Flickable.flickDeceleration
                var scaleFactor = units.gridUnit / 8;
                maximumFlickVelocity = maximumFlickVelocity * scaleFactor;
                flickDeceleration = flickDeceleration * scaleFactor;
            }
        }

        Scrollbar {
            flickableItem: dialogsListView
        }

        DelegateUtils {
            id: delegateUtils
        }

        TLInputPeer {
            id: inputPeer
            function setupPeer(peerId, peerType) {
                inputPeer.classType = peerType;
                if (TLInputPeer.TypeInputPeerChat === peerType) {
                    inputPeer.chatId = peerId;
                } else {
                    inputPeer.userId = peerId;
                }
            }
        }
    }
}
