import QtQuick 2.2
import Ubuntu.Components 1.1
import "../components"
import Telegram 0.1

TelegramPage {

    property alias firstName: firstNameTextField.text
    property alias lastName: lastNameTextField.text
    property alias error: errorTextField.text

    signal error(var id, var errorCode, string errorText)

    id: editProfilePage
    pageTitle: i18n.tr("Edit name");

    head.actions: [
            Action {
                iconName: "ok"
                text: i18n.tr("Done")
                onTriggered: saveProfile()
            },
            Action {
                iconName: "cancel"
                text: i18n.tr("Cancel")
                onTriggered: cancel()
            }
        ]

    body: Item {
        anchors.fill: parent
        anchors.margins: units.gu(2)

        TextField {
            id: firstNameTextField
            anchors {
                top: parent.top
                topMargin: units.gu(4)
                left: parent.left
                right: parent.right
            }
            validator: RegExpValidator {
                regExp: /[\w\s]+/
            }
            placeholderText: i18n.tr("First name (required)")
            inputMethodHints: Qt.ImhNoPredictiveText
            Keys.onReturnPressed: lastNameTextField.forceActiveFocus()
        }

        TextField {
            id: lastNameTextField
            anchors {
                top: firstNameTextField.bottom
                topMargin: units.gu(2)
                left: parent.left
                right: parent.right
            }
            validator: RegExpValidator {
                regExp: /[\w\s]+/
            }
            placeholderText: i18n.tr("Last name (optional)")
            inputMethodHints: Qt.ImhNoPredictiveText
            Keys.onReturnPressed: addContact()
        }

        Text {
            id: errorTextField
            anchors {
                top: lastNameTextField.bottom
                topMargin: units.gu(2)
                left: parent.left
                right: parent.right
            }
            horizontalAlignment: Text.AlignHCenter
            color: "red"
        }
    }

    onError: {
        error = i18n.tr("Could not update profile: ") + errorText;
    }

    Component.onCompleted: {
        telegramClient.error.connect(editProfilePage.error);
    }

    Component.onDestruction: {
        telegramClient.error.disconnect(editProfilePage.error);
    }

    function saveProfile() {
        Qt.inputMethod.commit();
        if (firstName.length > 0) {
            telegramClient.updateProfile(firstName, lastName);
            back();
        } else {
            error = i18n.tr("First name is required.");
        }
    }
}
